//The research was supported by the project  GACR GA15-03564S

//// Simulation of two dimensional static regression mixture 
////                  with dynamic data-dependent pointer
// -------------------------------------------------------------------------
// basic simulation examples
mode(0),funcprot(0);
exec("Intro.sce",-1);

nd=1500;                                // number of data
nc=3;                                   // number of components
ny=2;                                   // dimension of the output
kz=3;

// external variable zt 
zt=genz(kz,nd);

pp=list();
pp(1)=fnorm([0 0 1; 1 0 0; 0 1 0]+.05,2); 
pp(2)=fnorm([1 0 0; 0 1 0; 0 0 1]+.1,2);
pp(3)=fnorm([0 1 0; 0 0 1; 1 0 0]+.08,2); 

// here you can define your own component parameters:
ths=list();
ths(1)=[2 -2]';
ths(2)=[1 5]';
ths(3)=[8 1]';
ths(4)=[5 4]';
ths(5)=[-1 2]';
amp=.3;
cvs=list();
cvs(1)=amp*[1 .2;.2 2];
cvs(2)=amp*[2 -.9;-.9 1];
cvs(3)=amp*[1 -.3;-.3 1];
cvs(4)=amp*[2 .2;.2 3];
cvs(5)=amp*[3 .2;.2 3];
// --------------------------------------
ct(1)=1;                                // initial component
// SIMULATION
for t=2:nd
  ct(t)=sum(rand(1,1,'unif')>cumsum(pp(zt(t))(ct(t-1),:)))+1; // component generation
  yt(:,t)=ths(ct(t))+cvs(ct(t))*rand(ny,1,'norm');  // data generation
end

// generation of prior classified data
yi=[]; ni=5;                            // no. of data from each comp.
for i=1:nc
  j=find(ct==i);                        // data from component i
  ji=j(10+(1:ni));                      // selected from data sample
  yi=[yi [yt(:,ji); i*ones(1,ni)]];     // stored in yi
end

// RESULTS
set(figure(1),'position',[140 50 500 500])
set(gcf(),'background',8);
plot(yt(1,:),yt(2,:),'.')
title('Data')

set(figure(2),'position',[660 50 500 500])
set(gcf(),'background',8);
plot(ct,'.')
title('Clusters')
set(gca(),'data_bounds',[1 nd .9 nc+.1])

// SAVING THE DATA SAMPLE (the name is compulsory)
als=pp; ncs=nc;
save('_data/datRegEx','yt','zt','ct','yi','ths','cvs','als','ncs')
