//The research was supported by the project  GACR GA15-03564S

//// Simulation of two dimensional static regression mixture 
////                  with dynamic data-dependent pointer
// -------------------------------------------------------------------------
// basic simulation examples
mode(0),funcprot(0);
exec("Intro.sce",-1)

nd=1500;                                // number of data
nc=3;                                   // number of components
ny=2;                                   // dimension of the output

kz=3;                                   // no. of values of zt

// external variable zt 
zt=genz(kz,nd);

pp=list();
pp(1)=fnorm([0 0 1; 1 0 0; 0 1 0]+.05,2); 
pp(2)=fnorm([1 0 0; 0 1 0; 0 0 1]+.1,2);
pp(3)=fnorm([0 1 0; 0 0 1; 1 0 0]+.08,2); 

cp=list();
for i=1:kz
  cp(i)=fnorm(pp(i),2);
end

// here you can define your own component parameters:
nn=15;
ths=list();
//      L U
ths(1)=[0 3
        0 4];
ths(2)=[4 6
        5 9];
ths(3)=[7 10
        0 4.6];
n=size(ths(1),1);
nc=length(ths);
alS=fnorm(ones(1,nc));
nu=ones(1,nc);
al=fnorm(nu);
// --------------------------------------
ct(1)=1;                                // initial component
// SIMULATION
for t=2:nd
  ct(t)=sum(rand(1,1,'u')>cumsum(alS))+1;
  Us=ths(ct(t))(:,2); 
  Ls=ths(ct(t))(:,1);
  yt(:,t)=(Us-Ls).*rand(n,1,'u')+Ls;
end

// generation of prior classified data
yi=[]; ni=5;                            // no. of data from each comp.
for i=1:nc
  j=find(ct==i);                        // data from component i
  ji=j(10+(1:ni));                      // selected from data sample
  yi=[yi [yt(:,ji); i*ones(1,ni)]];     // stored in yi
end

// RESULTS
set(figure(1),'position',[140 50 500 500])
set(gcf(),'background',8);
plot(yt(1,:),yt(2,:),'.')
title('Data')

set(figure(2),'position',[660 50 500 500])
set(gcf(),'background',8);
plot(ct,'.')
title('Clusters')
set(gca(),'data_bounds',[1 nd .9 nc+.1])

// SAVING THE DATA SAMPLE (the name is compulsory)
als=cp; ncs=nc;
save('_data/datUniEx','yt','zt','ct','yi','ths','als','ncs')
