//The research was supported by the project  GACR GA15-03564S

// Mixture estimation for all model types                             
//  - components: tMix=1   normal regression model
//                tMix=2   categorical model
//                tMix=3   normal state-space model
//                tMix=4   exponential model
//                tMix=5   uniform model
//  - static data-dependent pointer model
// -------------------------------------------------------------------
exec("Intro.sce",-1), mode(0);

// CHOICE OF COMPONENT TYPE
tMix=1;       // 1=regr, 2=categ, 3=state, 4=exp, 5=uniform
      
// TASK SPECIFICATION
nd=600;                                 // number of data
fi=.99999;                              // exponential forgetting
///////////////////////////////////
                              
// DATA LOAD  (simulators are E1Sim, E2Sim, E3Sim)
select tMix
  case 1 then                           // - REGRESSION MIXTURE
    load('_data/datRegEx','yt','zt','ct','yi','ths','cvs','als','ncs');
  case 2 then                           // - DISCRETE MIXTURE
    load('_data/datDisEx','yt','zt','ct','yi','als','ps','ncs','kys');
  case 3 then                           // - STATE-SPACE MIXTURE
    load('_data/datStaEx','yt','zt','ct','als','ncs','M','A','F','Rw','Rv');
  case 4 then                           // - EXPONENTIAL MIXTURE
    load('_data/datExpEx','yt','zt','ct','yi','ths','as','als','ncs');
  case 5 then                           // - UNIFORM MIXTURE
    load('_data/datUniEx','yt','zt','ct','yi','ths','als','ncs');
end
nc=ncs; // no. of components is taken from simulation

// INITIALIZATION ------------------------------------------------------
// initialization of the pointer statistics
np=0;
nz=max(zt);
w=ones(nc,1)/nc; wy=w*ones(1,np+1);               // initial weights
nu=list(); al=list();
for i=1:nz
  nu(i)=5*eye(1,nc)+1*rand(1,nc,'unif');        // statistics 
  nu(i)=1e-5*ones(1,nc);     // XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  al(i)=fnorm(nu(i));                           // point estimates
end

// initialization of the component statistics
select tMix  
  case 1 then [Est,ka]=initReg(yi);               // ini.com.stats
  case 2 then mu=initDis(yi,max(yt,'c'));         // ini.com.stats
  case 3 then Rx=eye(2,2)*1e6; xx=zeros(2,1);     // ini. KF stats
  case 4 then [Est,ka]=initExp(yi,as);            // ini. Exp stats
  case 5 then [Est,ka]=initUni(yi);               // ini. Unif stats
end
if tMix==1, EstI=Est; end

// TIME LOOP ---------------------------------------------------------
printf('running ............|\n '),itime=0;
for t=2:nd                                                       //  1
  itime=itime+1; if itime>(nd-1)/20, mprintf('.'), itime=0; end
  // Computation of data-component "proximities"
  select tMix                                                    //  2
    case 1 then mp=predReg(nc,yt(:,t),Est);                      //  3
    case 2 then mp=predDis(nc,yt(:,t),mu);                       //  4
    case 3 then [mp,KF]=predSta(nc,xx,yt(:,t),M,A,F,Rw,Rv,Rx);   //  5
    case 4 then mp=predExp(nc,yt(:,t),Est);
    case 5 then mp=predUni(nc,yt(:,t),Est);
  end                                                            //  6
  if t<1 // number of steps without al
    wp=mp'; w=wp/sum(wp);                // without al           //  7
  else
    wp=mp'.*al(zt(t)); w=wp/sum(wp);     // with al             //  8
  end

  
  // Update of statitics and point estimates of parameters
  // - statistics of the pointer
  nu(zt(t))=nu(zt(t))+w;           // update of statistics       //  9
  al(zt(t))=fnorm(nu(zt(t)),2);    // point estimate of alpha    // 10

  wy=ones(1,nc)/nc;
  // - statistics of components
  select tMix                                                    // 12
    case 1 then [Est,yp,ka]=updtReg(w,yt(:,t),Est,ka,fi,wy);     // 13
    case 2 then [Est,yp,mu]=updtDis(w,yt(:,t),mu);               // 14
    case 3 then [xx,Rx,yp]=updtSta(w,KF);                        // 15
    case 4 then [Est,yp,ka]=updtExp(w,yt(:,t),Est,ka,fi,wy,as);  
    case 5 then [Est,yp,ka]=updtUni(w,yt(:,t),Est,ka,wy,.2,.1,3); 
                         // catch point; forget point; lag before forg.
  end                                                            // 17
  ypt(:,t+np)=yp;                  // store                      // 18
  wt(:,t)=w;                       // store (components)         // 19
  wyt(:,t)=wy;                     // store (prediction)         // 20

  // storing current results
  if tMix~=3                                                     // 21
  for i=1:nc                                                     // 22
    Res(i).th(:,t)=Est(i).th(:);   // i-th component center      // 23
  end                              // only for regression model  // 24
end                                                              // 25
end                                                              // 26

// end of TIME LOOP   ------------------------------------------------

[xxx Ect]=max(wt,'r');             // pt.est. - active comp.     // 27
[xxx Ecp]=max(wyt,'r');            // pt.est. - predic. comp.    // 28

if (max(ct)==max(Ect))
[q,T]=c2c(ct,Ect);
Tct=q(Ect);
if prod(sum(T,1))==0
  disp ' !!! At least one component is not found !!!'
end
end

select tMix
case 1 then exec('Results1.sce',-1)     // regression mixture
case 2 then exec('Results2.sce',-1)     // categ. mixture
case 3 then exec('Results3.sce',-1)     // state-space mixture
case 4 then exec('Results4.sce',-1)     // exp. mixture
case 5 then exec('Results5.sce',-1)     // unif. mixture
end
