//The research was supported by the project  GACR GA15-03564S

function KF=KFilt(xt,yt,A,C,F,Rw,Rv,Rx,ie)
  // [xt,ey,Rx,yp]=KFilt(xt,yt,A,C,F,Rw,Rv,Rx,ie)
  // Kalman filter for the state-space model in the following form
  //                        xt = A*xt + F
  //                        yp = C*xt 
  // ... first compute prediction and then filtration, i.e.  
  //               t-1|t-1  -->  t|t-1  -->  t|t
  // KF.xt    state estimate
  // KF.yp    predicted output
  // KF.Ry    output covariance
  // KF.ey    prediction error
  // KF.Rx    state covariance matrix
  // yt    data sample
  // A,C,F model parameters
  // Rw    state covariance
  // Rv    output covariance
  // ie    indicator ie=0 => only state prediction is computed
  //
  if argn(2)<9, ie=1; end

  // Prediction
  xt=A*xt+F;                           // time updt of state
  Rx=Rw+A*Rx*A';                       // time updt of state covariance

  // Filtration 
  yp=C*xt;                             // data prediction
  Ry=Rv+C*Rx*C';                       // output prediction covariance
  Ryy=Ry+1e-8*eye(Ry);
  Rx=Rx-Rx*C'*inv(Ryy)*C*Rx;            // data updt of  state covariance
  ey=yt-yp;                            // prediction error 
  if ie==0                             // skip filtration
    ey=zeros(ey);
  else
    KG=Rx*C'*inv(Rv);                    // Kalman gain
    xt=xt+KG*ey;                         // data updt of state
  end

  KF.xt=xt;
  KF.yp=yp;
  KF.Ry=Ry;
  KF.ey=ey;
  KF.Rx=Rx;
endfunction
